#!/bin/sh
###############################################################
#
# XTAL subprogram: Isosurf: Render Xtal compid.map file isosurfaces.
#
#       Copyright (C) 2000, The Xtal System, The University of
#             Western Australia, xtal@crystal.uwa.edu.au.
#  Distributed under the GNU General Public Licence as published by
#  the Free Software Foundation, in the hope that it will be useful
#  but WITHOUT ANY WARRANTY. See the GNU General Public License
#  for more details.
#
# DISCLAIMER: This is experimental software with known bugs.
#             Don't just wine that its broken. Fix it!
#             By the way the surface gradient calcn is broken
#             so all lighting is completely screwy.
#
# D. du Boulay ddb@R3401.rlem.titech.ac.jp  2001-10-23
#
###############################################################
# Ordinarily this script should be invoked from Xtal, however...
# for simple testing the next line restarts using wish \
exec wish "$0" "$@"

# manually set some initial default values
# should use some sensible algorithm really
set contourSettings(Dvalues) [list -.5 -.2 .2  .5 ];# contour levels
#set contourSettings(Dvalues) [list -4. -2. 2. 4. ];# contour levels
#set contourSettings(Dvalues) [list -40. -15. 15. 40.];# contour levels
set contourSettings(DrelVal) [list -0.8 -.3 .3  .8 ];# fraction of range
#set contourSettings(DrelVal) [list -0.8  .15 .3  .8 ];# fraction of range
set contourSettings(FixedVals) 0 ;# 0/1 aply relVal or Dvalues 

# Specify a colour gradient scheme for +ive and -ive contours
# The 4th coordinate is Opacity/transparency
set contourSettings(DmaxNeg) [list 1. 1. 1. 1.0 ] ; #  Maximum negative colour
set contourSettings(DminNeg) [list 1. 0. 0. 0.0 ] ; #  Minimum negative colour
set contourSettings(DmaxPos) [list 1. 1. 1. 1.0 ] ; #  Maximum positive colour
set contourSettings(DminPos) [list 0. 0. 1. 0.0 ] ; #  Minimum positive colour


# Load contour level overrides from tclsurfin.rc in users local directory.
#
if { [catch { source tclsurfin.rc}  res ]} {
#      puts  $res 
#      puts  "Can't open local contour override file: tclisosurf.rc"
#      puts  "using builtin defaults"
 }


# 3D Electron density: statically load our isosurface C module extension
# But first we need to load the static Togl extension package.
load "" Togl 
load "" Surfin 



#####################################################################
# Personally I wouldn't venture far beyond this point ...
#####################################################################

# The following procs are used for testing standalone in "wish"
# they should be overridden from within when running with tklib.
# (actually they are currently irrelevent)
if [string match *wish* [info nameofexecutable] ] {
   proc XtalBinding {top level id } { }
   proc xtal {args } { eval { canvas } $args }
}


# these items will be filled by tcl calls to the Isosurf.c module
set Density(min) 0
set Density(max) 0
# 9 components - cater for electric field gradient?
# 3 components - cater for electric field ? 
# 1 component  - cater for charge density - the only thing that works


# This variable is just a flag to let us know here when
# the Fortran side has done all necessary initializations 
# it is modified just before we hit the maineventloop
# Later we set a trace on it from the tcl side to take appropriate 
# initializations of our own.
set NewFortranData 0 ; # set from fortran 


set applyTo(Contour) [list Current "All contours"] 
set applyTo(DContour)         0 ; # current 
set applyTo(AContour)         $applyTo(DContour)
set applyTo(CContour)         $applyTo(AContour)


proc setContValue { id value } {
   global OPENGLWIN
   $OPENGLWIN cont $id value $value
}

proc delCont { id } {
   global OPENGLWIN
   $OPENGLWIN cont $id delete
}

proc ReloadSelf { } {
   source isosurf.tcl
}


# 
# Create the main window containing the togl widget 
#
proc setup {} {
   global OPENGLWIN
   global  NewFortranData
   global Density
    global Initialized
    if { [catch { $Initialized }]==0 } {
        destroy .contourW
        return
    }
    set Initialized 1

   global startx starty         # location of mouse when button pressed
   global xangle yangle
   global xangle0 yangle0
   global scale scale0

   wm title . "Xtal: Tk interface to SURFIN"

   ### Two frames:  top half and bottom half
   frame .f0 -relief raised
   frame .f1
   frame .f2

   menubutton .f0.b1 -text "File" -relief flat -menu  .f0.b1.m
   menubutton .f0.b2 -text "Edit" -relief flat -menu  .f0.b2.m
   menubutton .f0.b3 -text "Help" -relief flat -menu  .f0.b3.m
    menu .f0.b1.m
    .f0.b1.m add command -label "Print" -command PageDump
    .f0.b1.m add command -label "Quit" -command QuitFile
#    .f0.b1.m add command -label "Reload" -command ReloadSelf
    menu .f0.b2.m
    .f0.b2.m add command -label "Contours" -command ContourEdit
    .f0.b2.m add command -label "Lights" -command  LightSwitch
    menu .f0.b3.m
    .f0.b3.m add command -label "You're on your own!" 

    pack .f0.b1 .f0.b2  -side left -padx 2m  
    pack .f0.b3  -side right -padx 2m  
    pack .f0 -side top

    ### The OpenGL window
    set OPENGLWIN [togl .f1.view -width 530 -height 630 \
			-rgba true -double true -depth true ]
    pack $OPENGLWIN  -side left -fill both -expand true
    pack .f1 -side top -fill both -expand true


    label .f2.minlab  -text "Density  Min"
    entry .f2.mindens -width 12 -relief sunken \
                         -textvariable Density(min)
    label .f2.maxlab  -text "Max"
    entry .f2.maxdens -width 12 -relief sunken \
                         -textvariable Density(max)
    pack  .f2.minlab .f2.mindens .f2.maxlab .f2.maxdens -side left -fill y \
                             -expand false
    pack .f2 -side top -fill y 
    


    bind $OPENGLWIN <ButtonPress-1> {
	set startx %x
	set starty %y
    }

    bind $OPENGLWIN <B1-Motion> {
        set xangle [expr  (%x - $startx) / 3.0 ]
        set yangle [expr  (%y - $starty) / 3.0 ]
        $OPENGLWIN yrot $xangle
        $OPENGLWIN xrot $yangle
        set startx %x
        set starty %y
    }

    bind $OPENGLWIN <ButtonPress-2> {
	set startx %x
	set starty %y
	set scale0 $scale
    }

    bind $OPENGLWIN <B2-Motion> {
        set q [ expr ($starty - %y) / 400.0 ]
	set scale [expr $scale0 * exp($q)]
        $OPENGLWIN scale $scale
    }

    set xangle 0.0
    set yangle 0.0
    set scale 1.0

    # fortran source sets the following parameter non zero
    # just before we hit the event loop.
    trace variable NewFortranData w ResetContours 
}

#
#  Called in response to Fortran code having completed initializations
#  Currently all we do is set up some default contour levels before
#  handing back to the Fortran initiated event processing loop.
#
proc ResetContours {name1 name2 op } {
   global  NewFortranData
   set NewFortranData 0  ;  # reset this param
   InitContours
   InitLights
}

proc InitLights {} {
   global OPENGLWIN
   
   $OPENGLWIN light disable
   #$OPENGLWIN light model twoside
   #$OPENGLWIN light model ambience
   #$OPENGLWIN light model localview
   $OPENGLWIN light 0 enable
   $OPENGLWIN light 0 position   0.7 0.7 0.7    0.0
   $OPENGLWIN light 0 ambience    1.0 1.0 1.0  1.0
   $OPENGLWIN light 0 diffuse    1.0 1.0 1.0  1.0
   $OPENGLWIN light 0 specular   1.0 1.0 1.0  1.0

   $OPENGLWIN light 1 enable
   $OPENGLWIN light 1 position   0.6 -.6 0.6    1.0
   $OPENGLWIN light 1 ambience    1.0 1.0 1.0  1.0
   $OPENGLWIN light 1 diffuse    1.0 1.0 1.0  1.0
   $OPENGLWIN light 1 specular   1.0 1.0 1.0  1.0

   $OPENGLWIN light 2 enable
   $OPENGLWIN light 2 position   0.6 -.6 0.6    1.0
   $OPENGLWIN light 2 direction  0.6 -.6 0.6    1.0
   $OPENGLWIN light 2 cutoff 20. 
   $OPENGLWIN light 2 exponent 64.
   $OPENGLWIN light 2 ambience    1.0 1.0 1.0  1.0
   $OPENGLWIN light 2 diffuse    1.0 1.0 1.0  1.0
   $OPENGLWIN light 2 specular   1.0 1.0 1.0  1.0
   
}
#
# First get min and max densities from the C module
# Create some default contour levels and set appropriate settings
# Treat the zero level contour as special
# I think it is very difficult to know what is an internal or external
# surface of the zero contour.
#
proc InitContours {} {
   global OPENGLWIN
   global contourSettings
   global Density

   $OPENGLWIN  cont delete ;   # erase existing set of isosurfaces
   set mindensity  [ $OPENGLWIN mindensity ] ; # return a  9 elem array
   set maxdensity  [ $OPENGLWIN maxdensity ] ; # return a 9 elem  array 
   set min [lindex $mindensity 0]
   set max [lindex $maxdensity 0]
   set Density(min) [format "%.4f" [expr $min] ]
   set Density(max) [format "%.4f" [expr $max] ]

   # if min and max are both <0 or both >0 then potential probs here.
   if {[expr abs($min)] >[expr abs($max) ]} {
      set range [expr abs($min)] 
      #set dscal [expr $contourValues[
   } else { 
      set range [expr abs($max) ] 
   } 

#puts "range $range"

   if {$contourSettings(FixedVals) == 0} {
      set rellev  [lrange $contourSettings(ArelVal) 0 end ] ; # copy
      set contourSettings(Avalues)  [list ] ; # rewrite as empty list
      set contourSettings(Dvalues)  [list ] ; # rewrite as empty list
      for {set i 0} {$i<$contourSettings(Dlevels)} {incr i} {
            set val  [lindex $rellev $i ]
            lappend contourSettings(Avalues) [ expr $val * $range ]
            lappend contourSettings(Dvalues) [ expr $val * $range ]
      }
   }
            
       
   
    
   # preset some contour levels
   for {set i 0} {$i<$contourSettings(Dlevels)} {incr i} {
      set val  [lindex $contourSettings(Avalues) $i ]
      set ContourID [$OPENGLWIN cont new value $val ]  
      if {$val < 0.} {
        set val [expr -$val]
	set max $contourSettings(DmaxNeg) 
	set min $contourSettings(DminNeg) 
      } else {	
	set max $contourSettings(DmaxPos) 
	set min $contourSettings(DminPos) 
      }
      set fac1 [expr pow( $val / $range,0.7) ]
      set fac2 [expr pow( $val / $range,0.5) ]

      set col [list \
             [expr [lindex $max 0]*$fac1 + (1.-$fac1)*[lindex $min 0] ] \
             [expr [lindex $max 1]*$fac1 + (1.-$fac1)*[lindex $min 1] ] \
             [expr [lindex $max 2]*$fac1 + (1.-$fac1)*[lindex $min 2] ] \
             [expr [lindex $max 3]*$fac2 + (1.-$fac2)*[lindex $min 3] ] ]

      lappend contourSettings(Aids) $ContourID 
      eval $OPENGLWIN cont $ContourID ambience [lrange $col 0 3] 
      eval $OPENGLWIN cont $ContourID emmisive [lrange $col 0 3] 
      eval $OPENGLWIN cont $ContourID diffuse  [lrange $col 0 3] 
      eval $OPENGLWIN cont $ContourID specular  [lrange $col 0 3] 
      eval $OPENGLWIN cont $ContourID color   [lrange $col 0 3] 

      lappend contourSettings(ListAmbient)  $col
      lappend contourSettings(ListDiffuse)  $col
      lappend contourSettings(ListEmmision) $col
      lappend contourSettings(ListSpecular) $col
      lappend contourSettings(ListColor)    $col
      lappend contourSettings(ListShininess) $contourSettings(DshininessVal)
        
      lappend contourSettings(Ashininess)  $contourSettings(DshininessFlag)
      lappend contourSettings(Aambient)  $contourSettings(DambientFlag)
      lappend contourSettings(Adiffuse)  $contourSettings(DdiffuseFlag)
      lappend contourSettings(Aemmision) $contourSettings(DemmisionFlag)
      lappend contourSettings(Aspecular) $contourSettings(DspecularFlag)
      lappend contourSettings(Acolour)   $contourSettings(DcolourFlag)
      lappend contourSettings(Ainvoked)  $contourSettings(DinvokedFlag)
      lappend contourSettings(Afillmode)  $contourSettings(DfillmodeFlag)
      lappend contourSettings(Alinemode)  $contourSettings(DlinemodeFlag)
      lappend contourSettings(Aoverlay)  $contourSettings(DoverlayFlag)

      set contourSettings(Cactive) $i
      setContFlags 1
    }
}

# For each contour there are a number of flags 
# reflecting its attributes and how it should be displayed
# This procedure sets the flags for the currently active 
#
proc setContFlags { recalc } {
   global OPENGLWIN
   global contourSettings

   # This assumes a current active contour. Please don't dissapoint. 
   set indx $contourSettings(Cactive)
   set id   [lindex $contourSettings(Aids) $indx ]
   # duplicate the C-code flags
   set VALUERESET [expr  1 << 0 ] ; # require a complete recalc of surface
   set DISPCONTUR [expr  1 << 1 ] ; # simple on off display switch
   set COLORENDER [expr  1 << 2 ] 
   set DIFFRENDER [expr  1 << 3 ]
   set AMBIRENDER [expr  1 << 4 ]
   set EMMIRENDER [expr  1 << 5 ]
   set SPECRENDER [expr  1 << 6 ]
   set SHINRENDER [expr  1 << 7 ]
   set FILLRENDER [expr  1 << 8 ]
   set LINERENDER [expr  1 << 9 ]
   set OVERLAYER  [expr  1 << 10 ]

   #Flags as set for Current active contour level
   set flag [ expr   \
             ($VALUERESET * $recalc) |  \
             ($DISPCONTUR * $contourSettings(CinvokedFlag)) |  \
             ($COLORENDER * $contourSettings(CcolourFlag)) |  \
             ($DIFFRENDER * $contourSettings(CdiffuseFlag) ) |  \
             ($AMBIRENDER * $contourSettings(CambientFlag) ) |  \
             ($EMMIRENDER * $contourSettings(CemmisionFlag) ) |  \
             ($SPECRENDER * $contourSettings(CspecularFlag) ) |  \
             ($SHINRENDER * $contourSettings(CshininessFlag) ) |  \
             ($FILLRENDER * $contourSettings(CfillmodeFlag) ) |  \
             ($LINERENDER * $contourSettings(ClinemodeFlag) ) | \
             ($OVERLAYER  * $contourSettings(CoverlayFlag) )      ]


   $OPENGLWIN cont $id flags $flag
}




set  graphicAttributes(fillStyles) [list fill line ] 

#
# Create a window for editing contour attributes
#
proc ContourEdit {} {
   global OPENGLWIN
   global applyTo
   global contourSettings
   global graphicAttributes
 
   if { [winfo exists .contourW] } {
        set w .contourW
        if  [ winfo ismapped $w ] {
            wm withdraw  $w
        } else {
            ApplyToContourConfig $w init ;
            ContourConfig       $w init ;
            wm deiconify $w
        }
   } else {
      set w [ toplevel .contourW -borderwidth 10 ]
      wm title  $w "Isosurface Editing"
 
     set a [frame $w.fa]
     set b [frame $w.fb  -highlightthickness 1 -highlightbackground Red ]
     set c [frame $w.fc  ]
     set d [frame $w.fd  -highlightthickness 1 -highlightbackground Red ]
     set e [ frame $w.fe ]
     grid $a $b $d $e -sticky n
     grid $c -  -  - -sticky w
 
     listbox $w.conlist  \
             -xscrollcommand [list $w.xscroll set ] \
             -yscrollcommand [list $w.yscroll set ]
     scrollbar $w.xscroll -orient horizontal \
              -command [list $w.conlist xview ]
     scrollbar $w.yscroll -orient vertical \
              -command [list $w.conlist yview ]
 
     entry  $w.contour -text "contour"  -width 8 -relief sunken \
              -textvariable contourSettings(Tvalue)
 
 
     grid $w.contour     -         -in $a -sticky news
     grid $w.conlist  $w.yscroll  -in $a -sticky news
     grid $w.xscroll     x       -in $a -sticky news
 

     label $w.zzzz  -text "Contours" 
     button $w.new  -text "Add new"  -relief raised -pady 5 -padx 5 
     button $w.edit -text "Modify current" -relief raised  
     button $w.delete  -text "Delete current" -relief raised -pady 5 -padx 5

     grid $w.zzzz  -in $b -sticky news
     grid $w.new  -in $b -sticky news
     grid $w.edit  -in $b -sticky news
     grid $w.delete  -in $b -sticky news

 
     checkbutton $w.invoked -text "Invoke Contour" -command "setContFlags 0" \
                            -variable contourSettings(CinvokedFlag)
     checkbutton $w.fill -text "Fill Surface" -command "setContFlags 0" \
                            -variable contourSettings(CfillmodeFlag)
     checkbutton $w.line -text "Outline " -command "setContFlags 0" \
                            -variable contourSettings(ClinemodeFlag)
     checkbutton $w.overlay -text "Overlay"      -command "setContFlags 0" \
                            -variable contourSettings(CoverlayFlag)
     checkbutton $w.shininess -text "Shininess"   -command "setContFlags 0" \
                            -variable contourSettings(CshininessFlag)
     scale $w.shinyval -from 0 -to 1. -orient horizontal \
				-command "setContColour shininess"
     $w.shinyval configure  -resolution 0.01 -variable contourSettings(CshininessVal)

     frame $w.grid
     label  $w.grid.lab   -text "Resolution" 
     listbox $w.grid.step -height 1 -width 3 \
                 -yscrollcommand [list $w.grid.scroll set ]
     scrollbar $w.grid.scroll -orient vertical \
                 -command [list $w.grid.step yview ]
     pack $w.grid.lab $w.grid.step  -side left -fill x 
     pack  $w.grid.scroll -side left -fill both 

     frame $w.assign 
     label  $w.assign.text   -text "Apply To:"
     button $w.assign.to  -relief raised -pady 0 -underline 0 
     pack $w.assign.text  -side left -fill y  
     pack  $w.assign.to -side left -fill both  

     grid $w.invoked    -in $d -sticky nws
     grid $w.shininess  -in $d -sticky news
     grid $w.shinyval   -in $d -sticky nws
     grid $w.fill       -in $d -sticky nws
     grid $w.line       -in $d -sticky nws
     grid $w.overlay    -in $d -sticky nws
     grid $w.grid     -in $d -sticky nws
     grid $w.assign     -in $d -sticky nws
 
 
    # Specify contour polygon colours

    label $w.labcol -text Colour
    checkbutton $w.collab -text "Intrinsic" -command "setContFlags 0" \
                            -variable contourSettings(CcolourFlag)
    checkbutton $w.emmlab -text "Emmisive"  -command "setContFlags 0" \
                            -variable contourSettings(CemmisionFlag)
    checkbutton $w.amblab -text "Ambient"  -command "setContFlags 0" \
                            -variable contourSettings(CambientFlag)
    checkbutton $w.diflab -text "Diffuse"  -command "setContFlags 0" \
                            -variable contourSettings(CdiffuseFlag)
    checkbutton $w.speclab -text "Specular"  -command "setContFlags 0" \
                            -variable contourSettings(CspecularFlag)

    label $w.red  -text "Red"
    scale $w.colred -from 0 -to 1. -orient horizontal \
				-command "setContColour color"
    scale $w.emmred -from 0 -to 1. -orient horizontal \
				-command "setContColour emmision"
    scale $w.ambred -from 0 -to 1. -orient horizontal \
				-command "setContColour ambient"
    scale $w.difred -from 0 -to 1. -orient horizontal \
				-command "setContColour diffuse"
    scale $w.specred -from 0 -to 1. -orient horizontal \
				-command "setContColour specular"

    label $w.green  -text "Green"
    scale $w.colgreen -from 0 -to 1. -orient horizontal \
				-command "setContColour color"
    scale $w.emmgreen -from 0 -to 1. -orient horizontal \
				-command "setContColour emmision"
    scale $w.ambgreen -from 0 -to 1. -orient horizontal \
				-command "setContColour ambient"
    scale $w.difgreen -from 0 -to 1. -orient horizontal \
				-command "setContColour diffuse"
    scale $w.specgreen -from 0 -to 1. -orient horizontal \
				-command "setContColour specular"

    label $w.blue  -text "Blue"
    scale $w.colblue -from 0 -to 1. -orient horizontal \
				-command "setContColour color"
    scale $w.emmblue -from 0 -to 1. -orient horizontal \
				-command "setContColour emmision"
    scale $w.ambblue -from 0 -to 1. -orient horizontal \
				-command "setContColour ambient"
    scale $w.difblue -from 0 -to 1. -orient horizontal \
				-command "setContColour diffuse"
    scale $w.specblue -from 0 -to 1. -orient horizontal\
				-command "setContColour specular"

    label $w.opaq  -text "Opacity"
    scale $w.colopaq -from 0 -to 1. -orient horizontal \
				-command "setContColour color"
    scale $w.emmopaq -from 0 -to 1. -orient horizontal \
				-command "setContColour emmision"
    scale $w.ambopaq -from 0 -to 1. -orient horizontal \
				-command "setContColour ambient"
    scale $w.difopaq -from 0 -to 1. -orient horizontal \
				-command "setContColour diffuse"
    scale $w.specopaq -from 0 -to 1. -orient horizontal \
				-command "setContColour specular"

    $w.colred configure  -resolution 0.01 -variable contourSettings(CcolRed)
    $w.emmred configure  -resolution 0.01 -variable contourSettings(CemmRed)
    $w.ambred configure  -resolution 0.01 -variable contourSettings(CambRed)
    $w.difred configure  -resolution 0.01 -variable contourSettings(CdifRed)
    $w.specred configure -resolution 0.01 -variable contourSettings(CspecRed)
    $w.colgreen configure  -resolution 0.01 -variable contourSettings(CcolGreen)
    $w.emmgreen configure -resolution 0.01 -variable contourSettings(CemmGreen)
    $w.ambgreen configure -resolution 0.01 -variable contourSettings(CambGreen)
    $w.difgreen configure -resolution 0.01 -variable contourSettings(CdifGreen)
  $w.specgreen configure -resolution 0.01 -variable contourSettings(CspecGreen)
    $w.colblue configure  -resolution 0.01 -variable contourSettings(CcolBlue)
    $w.emmblue  configure -resolution 0.01 -variable contourSettings(CemmBlue)
    $w.ambblue  configure -resolution 0.01 -variable contourSettings(CambBlue)
    $w.difblue  configure -resolution 0.01 -variable contourSettings(CdifBlue)
    $w.specblue configure -resolution 0.01 -variable contourSettings(CspecBlue)
    $w.colopaq configure  -resolution 0.01 -variable contourSettings(CcolOpaq)
    $w.emmopaq  configure -resolution 0.01 -variable contourSettings(CemmOpaq)
    $w.ambopaq  configure -resolution 0.01 -variable contourSettings(CambOpaq)
    $w.difopaq  configure -resolution 0.01 -variable contourSettings(CdifOpaq)
    $w.specopaq configure -resolution 0.01 -variable contourSettings(CspecOpaq)
    

   grid $w.labcol $w.collab $w.emmlab $w.amblab $w.diflab $w.speclab   -in $c -sticky news
   grid $w.red $w.colred $w.emmred $w.ambred $w.difred $w.specred      -in $c -sticky news
   grid $w.green $w.colgreen $w.emmgreen $w.ambgreen $w.difgreen $w.specgreen -in $c -sticky news
   grid $w.blue $w.colblue $w.emmblue $w.ambblue $w.difblue $w.specblue -in $c -sticky news
   grid $w.opaq $w.colopaq $w.emmopaq $w.ambopaq $w.difopaq $w.specopaq -in $c -sticky news


 
 
     button $w.done    -text Done  \
           -command "wm withdraw  $w"
     button $w.apply   -text Apply   \
           -command "ContourConfig $w commit "
                     #ApplyToGraphConfig $w commit ;
     button $w.default -text Default  \
           -command "ContourConfig $w default "
     button $w.cancel  -text Cancel  \
           -command "ContourConfig $w cancel;  wm withdraw  $w"
 
 
     grid  $w.done  -in $e -sticky news
     grid  $w.apply  -in $e -sticky news
     grid  $w.default -in $e -sticky news
     grid  $w.cancel -in $e -sticky news
 
 
#      BuildPop $w.lstyle  $w.pop1  $graphicAttributes(fillStyles) \
#                            contourSettings(CfStyle)
#      bind $w.pop1 <ButtonRelease-1> "ContourConfig $w style "
# 
#      #BuildPop $w.lthick  $w.pop2  $graphicAttributes(lineThickness) \
#      #                      contourSettings(ClThick)
#      #bind $w.pop2 <ButtonRelease-1> "ContourConfig $w thickness "
#
# 
#      BuildPop $w.lmark  $w.pop3  $graphicAttributes(markStyles) \
#                            contourSettings(ClMark)
#      bind $w.pop3 <ButtonRelease-1> "ContourConfig $w mark "
# 
      BuildPop $w.assign.to  $w.pop4  $applyTo(Contour)  applyTo(CContour)
      bind $w.pop4 <ButtonRelease-1> "ApplyToContourConfig $w update "
 
 
      bind $w.contour <Return> "ContourConfig $w relabel"
 
 
      bind $w.conlist <ButtonRelease-1> \
      "set contourSettings(Aactive) \[lindex \[ $w.conlist curselection\] 0 \];\
         ContourConfig $w cancel ; ContourConfig $w init"
      bindtags $w.conlist [list Listbox $w.conlist all ]
 
 
      ContourConfig       $w init ;
      ApplyToContourConfig $w init ;
   }
 
}


set contourSettings(Dlevels) [llength $contourSettings(Dvalues) ]
set contourSettings(Alevels)  $contourSettings(Dlevels) 

set contourSettings(Dactive)  0 
set contourSettings(Aactive)  $contourSettings(Dactive) 
set contourSettings(Avalues)  $contourSettings(Dvalues) 
set contourSettings(ArelVal)  $contourSettings(DrelVal) 
set contourSettings(Aids)     [ list ]

set contourSettings(Ainvoked)   [ list ]  ; # list of flags for each contour
set contourSettings(Afillmode)  [ list ]  ; # list of flags for each contour
set contourSettings(Alinemode)  [ list ]  ; # list of flags for each contour
set contourSettings(Aoverlay)   [ list ]  ; # list of flags for each contour
set contourSettings(Aambient)   [ list ]  ; # list of flags for each contour
set contourSettings(Adiffuse)   [ list ]  ; # list of flags for each contour
set contourSettings(Aemmision)  [ list ]  ; # list of flags for each contour
set contourSettings(Aspecular)  [ list ]  ; # list of flags for each contour
set contourSettings(Aambient)   [ list ]  ; # list of flags for each contour
set contourSettings(Acolour)    [ list ]  ; # list of flags for each contour
set contourSettings(Ashininess) [ list ]  ; # list of flags for each contour

set contourSettings(ListAmbient) [ list ] ; # list of colours
set contourSettings(ListDiffuse) [ list ] ; # list of colours
set contourSettings(ListEmmision) [ list ] ; # list of colours
set contourSettings(ListSpecular) [ list ] ; # list of colours
set contourSettings(ListColour)   [ list ] ; # list of colours
set contourSettings(ListShininess) [list ] ; # list of values


set contourSettings(DinvokedFlag)   1
set contourSettings(DfillmodeFlag)   1
set contourSettings(DlinemodeFlag)   0
set contourSettings(DoverlayFlag)   0
set contourSettings(DcolourFlag)     1
set contourSettings(DdiffuseFlag)   1
set contourSettings(DambientFlag)   1
set contourSettings(DemmisionFlag)  1
set contourSettings(DspecularFlag)  1
set contourSettings(DfillmodeFlag)  1
set contourSettings(DlinemodeFlag)  1
set contourSettings(DshininessFlag) 0

set contourSettings(CinvokedFlag)   $contourSettings(DinvokedFlag)
set contourSettings(CfillmodeFlag)   $contourSettings(DfillmodeFlag)
set contourSettings(ClinemodeFlag)   $contourSettings(DlinemodeFlag)
set contourSettings(CoverlayFlag)   $contourSettings(DoverlayFlag)
set contourSettings(CcolourFlag)     $contourSettings(DcolourFlag)
set contourSettings(CdiffuseFlag)   $contourSettings(DdiffuseFlag) 
set contourSettings(CambientFlag)   $contourSettings(DambientFlag) 
set contourSettings(CemmisionFlag)  $contourSettings(DemmisionFlag)
set contourSettings(CspecularFlag)  $contourSettings(DspecularFlag)
set contourSettings(CfillmodeFlag)  $contourSettings(DfillmodeFlag)
set contourSettings(ClinemodeFlag)  $contourSettings(DlinemodeFlag)
set contourSettings(CshininessFlag) $contourSettings(DshininessFlag)

set contourSettings(DshininessVal)  0.0
set contourSettings(DemmRed)     0.5
set contourSettings(DambRed)     0.5
set contourSettings(DdifRed)     0.5
set contourSettings(DspecRed)     0.5
set contourSettings(DemmGreen)     0.5
set contourSettings(DambGreen)     0.5
set contourSettings(DdifGreen)     0.5
set contourSettings(DspecGreen)     0.5
set contourSettings(DemmBlue)     0.5
set contourSettings(DambBlue)     0.5
set contourSettings(DdifBlue)     0.5
set contourSettings(DspecBlue)     0.5
set contourSettings(DemmOpaq)     0.5
set contourSettings(DambOpaq)     0.5
set contourSettings(DdifOpaq)     0.5
set contourSettings(DspecOpaq)     0.5
set contourSettings(DcolrRed)      0.5
set contourSettings(DcolrGreen)      0.5
set contourSettings(DcolrBlue)      0.5
set contourSettings(DcolrOpaq)      0.5


set contourSettings(CemmRed)    $contourSettings(DemmRed)
set contourSettings(CambRed)    $contourSettings(DambRed)
set contourSettings(CdifRed)    $contourSettings(DdifRed)
set contourSettings(CspecRed)   $contourSettings(DspecRed)
set contourSettings(CemmGreen)  $contourSettings(DemmGreen)
set contourSettings(CambGreen)  $contourSettings(DambGreen)
set contourSettings(CdifGreen)  $contourSettings(DdifGreen)
set contourSettings(CspecGreen) $contourSettings(DspecGreen)
set contourSettings(CemmBlue)   $contourSettings(DemmBlue)
set contourSettings(CambBlue)   $contourSettings(DambBlue)
set contourSettings(CdifBlue)   $contourSettings(DdifBlue)
set contourSettings(CspecBlue)  $contourSettings(DspecBlue)
set contourSettings(CemmOpaq)   $contourSettings(DemmOpaq)
set contourSettings(CambOpaq)   $contourSettings(DambOpaq)
set contourSettings(CdifOpaq)   $contourSettings(DdifOpaq)
set contourSettings(CspecOpaq)  $contourSettings(DspecOpaq) 
set contourSettings(CcolrRed)   $contourSettings(DcolrRed) 
set contourSettings(CcolrGreen) $contourSettings(DcolrGreen) 
set contourSettings(CcolrBlue)  $contourSettings(DcolrBlue) 
set contourSettings(CcolrOpaq)  $contourSettings(DcolrOpaq) 

                            
proc setContColour { mode val } {
   global OPENGLWIN
   global contourSettings
   set indx $contourSettings(Cactive)
   set id   [lindex $contourSettings(Aids) $indx ] ; # not really default

   switch -exact -- $mode {
      ambient {
          set red   $contourSettings(CambRed)
          set green $contourSettings(CambGreen)
          set blue   $contourSettings(CambBlue)
          set opaq $contourSettings(CambOpaq)
         $OPENGLWIN cont $id ambience $red $green $blue $opaq
      }
      emmision {
          set red   $contourSettings(CemmRed)
          set green $contourSettings(CemmGreen)
          set blue   $contourSettings(CemmBlue)
          set opaq $contourSettings(CemmOpaq)
         $OPENGLWIN cont $id emmisive $red $green $blue $opaq
      }
      diffuse {
          set red   $contourSettings(CemmRed)
          set green $contourSettings(CemmGreen)
          set blue   $contourSettings(CemmBlue)
          set opaq $contourSettings(CemmOpaq)
         $OPENGLWIN cont $id diffuse $red $green $blue $opaq
      }
      specular {
          set red   $contourSettings(CemmRed)
          set green $contourSettings(CemmGreen)
          set blue  $contourSettings(CemmBlue)
          set opaq  $contourSettings(CemmOpaq)
         $OPENGLWIN cont $id specular $red $green $blue $opaq
      }
      color {
          set red   $contourSettings(CcolRed)
          set green $contourSettings(CcolGreen)
          set blue  $contourSettings(CcolBlue)
          set opaq  $contourSettings(CcolOpaq)
         $OPENGLWIN cont $id color $red $green $blue $opaq
      }
      shininess {
         $OPENGLWIN cont $id shininess  $contourSettings(CshininessVal)
      }
   }
}




proc ContourConfig {w mode} {
   global contourSettings
   global graphicAttributes
   global applyTo
 
   switch -exact -- $mode {
      init {
        set contourSettings(Cactive)    $contourSettings(Aactive)
        set c $contourSettings(Cactive)
    set contourSettings(Tvalue)  [lindex $contourSettings(Avalues) $c ]
    set contourSettings(CinvokedFlag) [lindex $contourSettings(Ainvoked) $c]
    set contourSettings(CfillmodeFlag) [lindex $contourSettings(Afillmode) $c]
    set contourSettings(ClinemodeFlag) [lindex $contourSettings(Alinemode) $c]
    set contourSettings(CoverlayFlag) [lindex $contourSettings(Aoverlay) $c]
    set contourSettings(CemmisionFlag) [lindex $contourSettings(Aemmision) $c]
    set contourSettings(CambientFlag) [lindex $contourSettings(Aambient) $c]
    set contourSettings(CdiffuseFlag) [lindex $contourSettings(Adiffuse) $c]
    set contourSettings(CspecularFlag) [lindex $contourSettings(Aspecular) $c]
    set contourSettings(CshininessFlag) [lindex $contourSettings(Ashininess) $c]
  set contourSettings(CshininessVal) [lindex $contourSettings(ListShininess) $c]

        set colour [lindex $contourSettings(ListEmmision) $c ]
           set contourSettings(CemmRed)     [lindex $colour 0 ]
           set contourSettings(CemmGreen)   [lindex $colour 1 ]
           set contourSettings(CemmBlue)    [lindex $colour 2 ]
           set contourSettings(CemmOpaq)    [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListAmbient) $c ]
           set contourSettings(CambRed)      [lindex $colour 0 ]
           set contourSettings(CambGreen)    [lindex $colour 1 ]
           set contourSettings(CambBlue)     [lindex $colour 2 ]
           set contourSettings(CambOpaq)     [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListDiffuse) $c ]
           set contourSettings(CdifRed)     [lindex $colour 0 ]
           set contourSettings(CdifGreen)   [lindex $colour 1 ]
           set contourSettings(CdifBlue)    [lindex $colour 2 ]
           set contourSettings(CdifOpaq)    [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListSpecular) $c ]
           set contourSettings(CspecRed)   [lindex $colour 0 ]
           set contourSettings(CspecGreen) [lindex $colour 1 ]
           set contourSettings(CspecBlue)  [lindex $colour 2 ]
           set contourSettings(CspecOpaq)  [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListColor) $c ]
           set contourSettings(CcolRed)     [lindex $colour 0 ]
           set contourSettings(CcolGreen)   [lindex $colour 1 ]
           set contourSettings(CcolBlue)    [lindex $colour 2 ]
           set contourSettings(CcolOpaq)    [lindex $colour 3 ]

         $w.conlist delete 0 end ;       # delete originals
         eval { $w.conlist insert end } $contourSettings(Dvalues)
      }
      cancel {
        set c $contourSettings(Cactive)
    set contourSettings(Tvalue)  [lindex $contourSettings(Avalues) $c ]
    set contourSettings(CinvokedFlag) [lindex $contourSettings(Ainvoked) $c]
    set contourSettings(CfillmodeFlag) [lindex $contourSettings(Afillmode) $c]
    set contourSettings(ClinemodeFlag) [lindex $contourSettings(Alinemode) $c]
    set contourSettings(CoverlayFlag) [lindex $contourSettings(Aoverlay) $c]
    set contourSettings(CemmisionFlag) [lindex $contourSettings(Aemmision) $c]
    set contourSettings(CambientFlag) [lindex $contourSettings(Aambient) $c]
    set contourSettings(CdiffuseFlag) [lindex $contourSettings(Adiffuse) $c]
    set contourSettings(CspecularFlag) [lindex $contourSettings(Aspecular) $c]
    set contourSettings(CshininessFlag) [lindex $contourSettings(Ashininess) $c]
  set contourSettings(CshininessVal) [lindex $contourSettings(ListShininess) $c]

        set colour [lindex $contourSettings(ListEmmision) $c ]
           set contourSettings(CemmRed)     [lindex $colour 0 ]
           set contourSettings(CemmGreen)   [lindex $colour 1 ]
           set contourSettings(CemmBlue)    [lindex $colour 2 ]
           set contourSettings(CemmOpaq)    [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListAmbient) $c ]
           set contourSettings(CambRed)      [lindex $colour 0 ]
           set contourSettings(CambGreen)    [lindex $colour 1 ]
           set contourSettings(CambBlue)     [lindex $colour 2 ]
           set contourSettings(CambOpaq)     [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListDiffuse) $c ]
           set contourSettings(CdifRed)     [lindex $colour 0 ]
           set contourSettings(CdifGreen)   [lindex $colour 1 ]
           set contourSettings(CdifBlue)    [lindex $colour 2 ]
           set contourSettings(CdifOpaq)    [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListSpecular) $c ]
           set contourSettings(CspecRed)   [lindex $colour 0 ]
           set contourSettings(CspecGreen) [lindex $colour 1 ]
           set contourSettings(CspecBlue)  [lindex $colour 2 ]
           set contourSettings(CspecOpaq)  [lindex $colour 3 ]
        set colour [lindex $contourSettings(ListColor) $c ]
           set contourSettings(CcolRed)     [lindex $colour 0 ]
           set contourSettings(CcolGreen)   [lindex $colour 1 ]
           set contourSettings(CcolBlue)    [lindex $colour 2 ]
           set contourSettings(CcolOpaq)    [lindex $colour 3 ]

        # undo modified display
        setContFlags 0  
        setContColour color dummy
        setContColour emmision dummy
        setContColour ambient dummy
        setContColour diffuse dummy
        setContColour specular dummy
        setContColour shininess dummy
      }
      commit {
        set contourSettings(Aactive)    $contourSettings(Cactive)
        set c $contourSettings(Cactive)
        set contourSettings(Ainvoked) [ lreplace $contourSettings(Ainvoked) \
                            $c $c $contourSettings(CinvokedFlag) ]
        set contourSettings(Afillmode) [lreplace $contourSettings(Afillmode) \
                            $c $c $contourSettings(CfillmodeFlag) ]
        set contourSettings(Alinemode) [lreplace $contourSettings(Alinemode) \
                            $c $c $contourSettings(ClinemodeFlag) ]
        set contourSettings(Aoverlay) [lreplace $contourSettings(Aoverlay) \
                            $c $c $contourSettings(CoverlayFlag) ]
        set contourSettings(Aemmision) [ lreplace $contourSettings(Aemmision) \
                            $c $c $contourSettings(CemmisionFlag) ]
        set contourSettings(Aambient) [ lreplace $contourSettings(Aambient) \
                            $c $c $contourSettings(CambientFlag) ]
        set contourSettings(Adiffuse) [ lreplace $contourSettings(Adiffuse) \
                            $c $c $contourSettings(CdiffuseFlag) ]
        set contourSettings(Aspecular) [ lreplace $contourSettings(Aspecular) \
                            $c $c $contourSettings(CspecularFlag) ]
        set contourSettings(Ashininess) [lreplace $contourSettings(Ashininess) \
                            $c $c $contourSettings(CshininessFlag) ]
        set contourSettings(ListShininess) [lreplace \
         $contourSettings(ListShininess) $c $c $contourSettings(CshininessVal) ]
 
        set contourSettings(ListEmmision) [ lreplace \
           $contourSettings(ListEmmision) $c $c [list \
           $contourSettings(CemmRed)  $contourSettings(CemmGreen) \
           $contourSettings(CemmBlue)  $contourSettings(CemmOpaq)  ]  ]
        set contourSettings(ListAmbient) [ lreplace \
           $contourSettings(ListAmbient) $c $c [list \
           $contourSettings(CambRed)  $contourSettings(CambGreen) \
           $contourSettings(CambBlue)  $contourSettings(CambOpaq)  ]  ]
        set contourSettings(ListDiffuse) [ lreplace \
           $contourSettings(ListDiffuse) $c $c [list \
           $contourSettings(CdifRed)  $contourSettings(CdifGreen) \
           $contourSettings(CdifBlue)  $contourSettings(CdifOpaq)  ]  ]
        set contourSettings(ListSpecular) [ lreplace \
           $contourSettings(ListSpecular) $c $c [list \
           $contourSettings(CspecRed)  $contourSettings(CspecGreen) \
           $contourSettings(CspecBlue)  $contourSettings(CspecOpaq)  ]  ]
        set contourSettings(ListColor) [ lreplace \
           $contourSettings(ListColor) $c $c [list \
           $contourSettings(CcolRed)  $contourSettings(CcolGreen) \
           $contourSettings(CcolBlue)  $contourSettings(CcolOpaq)  ]  ]
            
      }
 
   }
 
   #$w.lstyle  configure -relief raised \
   #    -text [ lindex $graphicAttributes(lineStyles) $contourSettings(ClStyle) ]
   #$w.lthick  configure -relief raised \
   #    -text [ lindex $graphicAttributes(lineThickness) \
   #               [expr int($contourSettings(ClThick)) ]  ]
   #$w.lmark  configure -relief raised \
   #    -text [ lindex $graphicAttributes(markStyles) $contourSettings(ClMark)  ]
 
   #set  contourSettings(Tcontour) \
   #    [lindex $contourSettings(Traces) $contourSettings(Cactive)]
# 
#   set contourSettings(TyOffset) [format "%.4f" [expr $contourSettings(CyOffset)]]
 

}

set lightSettings(state)  0
proc  LightSwitch {} {
    global OPENGLWIN
    global lightSettings
    if {$lightSettings(state) == 0 } {
          # puts "tcl enabling "
	$OPENGLWIN light enable
        set lightSettings(state) 1
    } else {
          # puts "tcl disabling "
	$OPENGLWIN light disable
        set lightSettings(state) 0
    }
}


proc QuitFile {} {
   set w [ toplevel .quit -borderwidth 10 ]
 
   label $w.quest -bitmap questhead -padx 2 ;  #use built in question head icon
   label $w.lab -text  "OK to quit?"  -padx 0
   frame $w.ok -borderwidth 2 -relief sunken
 
# No need to set a binding on this because the destruction of
# all canvases by virtue of destruction of . ensures the
# C API xtalSegment flag is set to -1  (signalling termination)
   button $w.ok.b -text OK  -command "destroy $w ; destroy . "
   pack $w.ok.b
   button $w.cancel -text cancel \
        -command "  destroy $w "
   pack $w.quest $w.lab -side top
   pack $w.ok $w.cancel -side left -padx 5 -pady 5
 
   focus $w         ;  # get mouse focus
   grab $w          ;  # grab the keyboard also
   tkwait window $w ;  # wait for window to be destroyed
   # destruction of this window should release the grab
}


set pageDumpCount 0
proc PageDump {  } {
   global pageDumpCount
 

   set w [ toplevel .pagedump  -borderwidth 10 ]
 
   label $w.quest -bitmap questhead -padx 2 ;  #use built in question head icon
   if {$pageDumpCount <= 0 } {
      set text \
"This will create/overwrite a new surfin.eps plot file\n with the current graphics window.\n Do you wish to proceed?"
   } else {
"This will create/overwrite a new surfin.eps plot file\n with the current graphics window.\n Do you wish to proceed?"
  }
 
   label $w.lab -text  $text -padx 0
   frame $w.ok -borderwidth 2 -relief sunken
 
   button $w.ok.b -text OK \
       -command "incr pageDumpCount ; Print surfin.eps rgb ; destroy $w" ;
   pack $w.ok.b
   button $w.cancel -text cancel -command "  destroy $w "
   pack $w.quest $w.lab -side top
   pack $w.ok $w.cancel -side left -padx 5 -pady 5
 
   awaitDestruction $w
}

proc Print {filename rgb} {
     global OPENGLWIN
     $OPENGLWIN epsfile $filename $rgb
}

proc ApplyToContourConfig {w mode} {
   global applyTo
#   global updateFlag
 
   switch -exact -- $mode {
      init {
          set applyTo(CContour)         $applyTo(AContour)
      }
      commit {
      }
      update {  # no-op as its all validated below
      }
      default {          
         set applyTo(CContour)         $applyTo(DContour)
      }
   }
 
#  set the current button mode
   set mode       [expr int($applyTo(CContour)) ]
   $w.assign.to configure -relief raised \
              -text [lindex $applyTo(Contour)  $mode]
}


#############################################################
#
#  General utility functions
#  -------------------------
#
#############################################################

proc BuildPop {button pop range  varname } {
   upvar #0 $varname  var
   menu $pop  -tearoff false
   set row 0
   foreach ptype $range {
        $pop add radio -hidemargin false -variable $varname \
            -label $ptype -value $row
        incr row
   }

   bind $button <ButtonPress-1> "ExGenPopup $button $pop \$$varname "
   # Reorder bindtags so Menu bindings fire first 
   # this ensures that the radio variable is updated before calling 
   # ExGenPopup
   bindtags $pop [list Menu $pop all ]
}

proc ExGenPopup { buttn  pop  val } {
   set w2   [winfo reqwidth  $pop ] 
   set h2   [winfo reqheight $pop ]
   set kids  [$pop index last ]
   set xr   [ winfo rootx $buttn ]
   set yr   [ winfo rooty $buttn ]
   set xn   [expr round ($xr + 0.5 * $w2) ] 
   set yn   [expr $yr + round ($h2 / ( $kids + 1 )/ 2.) ]
   catch { wm deiconify $pop } ;# if it does(nt) exist already
   tk_popup $pop  $xn $yn  [expr int( $val) ]
}

proc awaitDestruction {w } {
   set old [focus]
   catch {tkwait visibility $w };  # wait for window to be destroyed
   focus $w           ;  # get mouse focus
   catch {grab $w }   ;  # grab the keyboard also
   raise $w          ;   # put it on top
   tkwait window $w ;  # wait for window to be destroyed
   # destruction of this window should release the grab
   focus $old
}

# Execution starts here!
setup

